#version 130

/*
    ==== COMPOSITE:COLOR_BALANCE ====
    - Color balance controls
    - Vibrance adjustment
*/

#define GENERAL_BRIGHTNESS 1.0
#define WHITE_BRIGHTNESS 1.0  // Default white point brightness
#define BRIGHT_CONTRAST 1.0       // Default contrast for brighter colors
#define DARK_CONTRAST 1.0           // Default contrast for darker colors
#define SATURATION 1.0          // Default saturation
#define PALE_SATURATION 1.0    // Default saturation for paler colors
#define VIBRANCE 1.0          // Vibrance enhancement factor

uniform sampler2D colortex0;
varying vec2 texcoord;

// Function to adjust contrast
vec3 adjustContrast(vec3 color, float contrast) {
    return (color - 0.5) * contrast + 0.5;
}

// Function to adjust saturation
vec3 adjustSaturation(vec3 color, float saturation) {
    float luminance = dot(color, vec3(0.2126, 0.7152, 0.0722));
    return mix(vec3(luminance), color, saturation);
}

// Function to adjust vibrance
vec3 adjustVibrance(vec3 color, float vibrance) {
    float maxChannel = max(max(color.r, color.g), color.b);
    float minChannel = min(min(color.r, color.g), color.b);
    float saturation = maxChannel - minChannel;
    return mix(vec3(dot(color, vec3(0.299, 0.587, 0.114))), 
              color, 
              1.0 + (vibrance * (1.0 - saturation)));
}

void main() {
    // Sample original color
    vec3 color = texture2D(colortex0, texcoord).rgb;

    // Apply general brightness
    color *= GENERAL_BRIGHTNESS;

    // Apply white brightness to bright areas
    float luminance = dot(color, vec3(0.2126, 0.7152, 0.0722));
    color *= mix(1.0, WHITE_BRIGHTNESS, smoothstep(0.7, 1.0, luminance));

    // Apply contrast separately for bright and dark areas
    vec3 brightColors = step(0.5, color) * color;
    vec3 darkColors = (1.0 - step(0.5, color)) * color;
    
    brightColors = adjustContrast(brightColors, BRIGHT_CONTRAST);
    darkColors = adjustContrast(darkColors, DARK_CONTRAST);
    
    color = brightColors + darkColors;

    // Apply saturation adjustments
    float colorIntensity = max(max(color.r, color.g), color.b);
    color = adjustSaturation(color, mix(PALE_SATURATION, SATURATION, colorIntensity));

    // Apply vibrance
    color = adjustVibrance(color, VIBRANCE);

    // Ensure colors stay in valid range
    color = clamp(color, 0.0, 1.0);

    gl_FragColor = vec4(color, 1.0);
}
