// composite.fsh
#version 130
#include "/lib/util/dither.glsl"
#include "/settings.glsl"

varying vec2 TexCoords;
uniform float viewWidth, viewHeight, frameTimeCounter;
uniform vec3 cameraPosition, previousCameraPosition;
uniform mat4 gbufferPreviousProjection, gbufferProjectionInverse;
uniform mat4 gbufferPreviousModelView, gbufferModelViewInverse;
uniform sampler2D colortex0;
uniform sampler2D depthtex1;

const float DEPTH_THRESHOLD = 0.96;
const float MAX_BLUR_DISTANCE = 0.56; // Maximum blur distance as percentage of screen

float random(vec2 st) {
    return fract(sin(dot(st.xy, vec2(12.9898, 78.233))) * 43758.5453123);
}

float getLinearDepth(float depth) {
    return (2.0 * near * far) / (far + near - depth * (far - near));
}

vec2 calculateVelocity(float depth) {
    // Current position in screen space
    vec4 currentPosition = vec4(TexCoords * 2.0 - 1.0, depth * 2.0 - 1.0, 1.0);
    
    // Transform to world space
    vec4 viewPos = gbufferProjectionInverse * currentPosition;
    viewPos = gbufferModelViewInverse * viewPos;
    viewPos /= viewPos.w;
    
    // Calculate previous position
    vec3 cameraOffset = cameraPosition - previousCameraPosition;
    vec4 previousPosition = gbufferPreviousProjection * gbufferPreviousModelView * (viewPos + vec4(cameraOffset, 0.0));
    previousPosition /= previousPosition.w;
    
    // Calculate velocity
    vec2 velocity = (currentPosition - previousPosition).xy;
    
    // Limit maximum blur distance
    float velocityLength = length(velocity);
    if (velocityLength > MAX_BLUR_DISTANCE) {
        velocity *= MAX_BLUR_DISTANCE / velocityLength;
    }
    
    return velocity * MOTION_BLUR_STRENGTH;
}

vec3 MotionBlur(vec3 color, float depth, float dither) {
    if (depth <= DEPTH_THRESHOLD) return color;
    
    vec2 velocity = calculateVelocity(depth);
    float velocityLength = length(velocity);
    
    // Skip if movement is negligible
    if (velocityLength < 0.001) return color;
    
    vec3 mblur = color;
    float totalWeight = 1.0;
    
    // Adaptive sampling based on velocity
    int samples = int(mix(float(MOTION_BLUR_SAMPLES) * 0.5, float(MOTION_BLUR_SAMPLES), 
                         smoothstep(0.0, MAX_BLUR_DISTANCE, velocityLength)));
    
    for (int i = 1; i < samples; i++) {
        float t = (float(i) + dither) / float(samples);
        
        // Progressive offset
        vec2 offset = velocity * (t - 0.5);
        vec2 sampleCoord = TexCoords + offset;
        
        // Check if sample is within screen bounds
        if (sampleCoord.x < 0.0 || sampleCoord.x > 1.0 || 
            sampleCoord.y < 0.0 || sampleCoord.y > 1.0) continue;
        
        // Sample color and depth
        vec3 sampleColor = texture2D(colortex0, sampleCoord).rgb;
        float sampleDepth = texture2D(depthtex1, sampleCoord).x;
        
        // Depth-based weight
        float depthDiff = abs(depth - sampleDepth);
        float depthWeight = exp(-depthDiff * 100.0);
        
        // Distance-based weight
        float distWeight = 1.0 - abs(t - 0.5) * 2.0;
        
        // Combine weights
        float weight = depthWeight * distWeight;
        
        mblur += sampleColor * weight;
        totalWeight += weight;
    }
    
    return mblur / totalWeight;
}

void main() {
    vec3 color = texture2D(colortex0, TexCoords).rgb;
    
    #if MOTION_BLUR_ENABLED == 1
        float depth = texture2D(depthtex1, TexCoords).x;
        float dither = Bayer64(gl_FragCoord.xy) + random(TexCoords * frameTimeCounter);
        color = MotionBlur(color, depth, dither);
    #endif
    
    gl_FragData[0] = vec4(color, 1.0);
}
